# stats.py
#
#

""" statistics on suicide. """

from meds.utils.tijd import elapsed, to_time, today
from meds.utils.misc import fields
from meds.clock import Repeater
from meds.object import Object
from meds.kernel import kernel
from meds.event import Event
from meds.cfg import cfg

from campagne.defines import omschrijving, soort, gemeenten, tags, urls

import logging
import random
import time
import sys

#startdate = "2013-01-17 00:00:00"
startdate = "2016-01-01 00:00:00"
starttime = to_time(startdate)
source = "http://pypi.python.org/pypi/meds"
factor = 2.5

nrsec = Object()
nrsec.dag = 24 * 60 * 60.0
nrsec.jaar = 365 * nrsec.dag
nrsec.weekend = 2 / 7 * (24 * 60 * 60.0 * 365) / 52
nrsec.avond = 16 / 24 * (24 * 60 * 60.0)   

def getnumber(modname, name):
    mod = sys.modules[modname]
    for n in dir(mod):
        obj = getattr(mod, n, None)
        if obj:
            try: return int(obj.get(name))
            except: pass
    return 0

def seconds(name, period="dag"):
    nr = getnumber(__name__, name)
    return nrsec.get(period) / float(nr)

def per(delta, period="dag"):
    return delta / nrsec.get(period)

def nr(name, delta):
    nr = getnumber(__name__, name)
    return per(delta) * nr

def numbers(name):
    if period == "dag": delta = time.time() - today() ; return 
    else: delta = time.time() - starttime ; return delta/seconds(nr, period)

def init(event):
    for key, obj in objs.items():
        if not cfg.main.all and key not in cfg.main.init: continue
        for name, val in obj.items():
            if name in fields(perdag): period = "dag" ; delta = time.time() - today()
            else: period = "jaar" ; delta = time.time() - starttime
            sec = seconds(name, period)
            if sec:
                repeater = Repeater(sec, stats, event, name=name)
                kernel.launch(repeater.start)

def stop(kernel):
    for key, obj in objs.items():
        for k in obj.keys():
            kernel.kill(k)

def stats(event, **kwargs):
    global display
    e = Event(kwargs)
    name = event.rest or e.name 
    if not name: event.reply(",".join(sorted(fields(objs)))) ; return
    if name in fields(perdag): period = "dag" ; delta = time.time() - today()
    else: period = "jaar" ; delta = time.time() - starttime
    txt = "%s #%s" % (name.upper(), int(nr(name, delta)))
    if name in omschrijving: txt += " %s" % omschrijving.get(name)
    txt += " elke %s" % elapsed(seconds(name, period))
    if name in soort: txt += " door een %s" % soort.get(name, "slachtoffer")
    txt += " bijv. in %s" % random.choice(gemeenten)
    if name in tags: txt += " %s" % tags.get(name)
    if cfg.stats.showurl and name in urls: txt += " - %s" % urls.get(name)
    kernel.announce(txt)

#:
times = Object()
times.weekend = 2 / 7 * (24 * 60 * 60.0 * 365) / 52
times.avond = 16 / 24 * (24 * 60 * 60.0)
times.dag = 24 * 60 * 60.0
times.jaar = 365 * 24 * 60 * 60.0


#:
rechter = Object()
rechter.ibs = 7964
rechter.rm = 14902
rechter.vwm = 6163
rechter.mvv = 3966
rechter.vlm = 4699
rechter.mev = 65
rechter.ob = 0
rechter.zb = 9

#:
drugs = Object()
drugs.speed = 20000
drugs.cocaine = 50000
drugs.alcohol = 400000
drugs.wiet = 500000

#:
recepten = Object()
recepten.antipsychotica = 150000
recepten.antidepresiva = 600000
recepten.slaapmiddel = 1000000

#:
demografie = Object()
demografie.ambulant = 792000
demografie.verslaving = 13000
demografie.schizofrenie = 9800
demografie.depressie = 9600
demografie.verslaafden = 2074278
demografie.arbeidshandicap = 103000
demografie.huisartsen = 11345
demografie.zorgmijder = 24000

#:
cijfers = Object()
cijfers.opnames = 22866
cijfers.crisis = 150000
cijfers.oordeel = 150000
cijfers.pogingen = 94000
cijfers.incidenten = 66000 * factor
cijfers.poh = 1300000
cijfers.vergiftigingen = 25262
cijfers.overlast = 18000
cijfers.insluiting = 240000
cijfers.aangiftes = 134000
cijfers.suicide = 1854
cijfers.burenoverlast = 12000
cijfers.uitzetting = 5900
cijfers.volwassendoop = 500
cijfers.tumor = 12000
cijfers.detox = 65654
cijfers.acuut = 8000
cijfers.spoedeisendpoging = 14000
cijfers.weguitkliniek = 2539
cijfers.bewindvoering = 295000

#:
medicijnen = Object()
medicijnen.amitriptyline = 189137
medicijnen.paroxetine = 186028
medicijnen.citalopram = 154620
medicijnen.oxazepam = 133608
medicijnen.venlafaxine = 112000
medicijnen.mirtazapine = 110742
medicijnen.quetiapine = 84414
medicijnen.diazepam = 72000
medicijnen.sertraline = 68000
medicijnen.haloperidol = 59825

#:
oordeel = Object()
oordeel.verwijs = cijfers.crisis * 0.85 
oordeel.uitstroom = cijfers.crisis * 0.05
oordeel.opname = cijfers.crisis * 0.10

#:
alarm = Object()
alarm.politie = 0.15 * cijfers.crisis
alarm.hap = 0.40 * cijfers.crisis
alarm.keten = 0.45 * cijfers.crisis

#:
dbc = Object()
dbc.middelgebondenstoornissen = 33060
dbc.somatoformestoornissen = 21841
dbc.cognitievestoornissen = 25717
dbc.angststoornissen = 54458
dbc.aanpassingsstoornissen = 43079
dbc.depressievestoornissen = 102361
dbc.eetstoornissen = 8688
dbc.restgroepdiagnose = 16996
dbc.ontbrekendeprimairediagnose = 3030
dbc.andereproblemenredenvoorzorg = 49286
dbc.schizofrenieenanderepsychotischestoornissen = 6798
dbc.bipolairestoornissen = 3569
dbc.posttraumatischestressstoornis = 24716
dbc.persoonlijkheidsstoornissen = 36574
dbc.adhd = 25951
dbc.gedrag = 1176
dbc.kindertijdoverig = 1035
dbc.autismespectrum = 9436

#:
halfwaarde = Object()
halfwaarde.zyprexa = 30
halfwaarde.abilify = 75
halfwaarde.haldol = 30
halfwaarde.alprazolam = 11
halfwaarde.orap = 55
halfwaarde.paracetamol = 2.5
halfwaarde.lorazepam = 12
halfwaarde.paroxetine = 21 
halfwaarde.citalopram = 35
halfwaarde.oxazepam = 8.2
halfwaarde.quetiapine = 6
halfwaarde.diazepam = 100
halfwaarde.wiet = 7

#:
dodelijk = Object()
dodelijk.abilify = 500

#:
suicide = Object()
suicide.suicide = cijfers.suicide

#: 
perdag = Object()
perdag.medicijnen = medicijnen
perdag.drugs = drugs

#:
objs = Object()
objs.rechter = rechter
objs.demografie = demografie
objs.cijfers = cijfers
objs.dbc = dbc
objs.oordeel = oordeel
objs.medicijnen = medicijnen
objs.recepten = recepten
objs.drugs = drugs
objs.alarm = alarm
objs.perdag = perdag
